(*********************************************************************

	Cool`GenePro`  package


 *********************************************************************)


(* :Author: Thomas Cool *)
(* :Copyright: Copyright 1995 Thomas Cool *)
(* :Package Version: 2.0,  June 1997.*)
(* :Mathematica Version: 3.0 *)
(* :Keywords:  Genetic Programming  *)
(* :Limitation: *)
(* :License: use of this software file requires a license, usually available
      by payment of $259 to Swift account ING.B.NL.2A 3736767 to the name of 
      Thomas Cool, Rotterdamsestraat 69, 2586 GH Scheveningen, Holland.
      Please inquire about the cost of file packs. See the License[] function. *)
(* :Comment: This package was inspired by a notebook "Genetic Programming with Mathematica", by Jonathan Kleid, Son Of a Bit Software 1994, available from MathSource. Unfortunately, Kleid's electronic address was not functioning, so I found myself unable to discuss with him the possibility of simply copying some of his routines, and thus I have written my own. I found that the problem very much dictates the programming structure: (i) an initial population, (ii) scoring on a criterion, (iii) breeding, (iv) iterate. I noted errors in Kleid's notebook, on crossover, his "doCheck" subroutine, and fitness selection. Since I started with Kleid's notebook myself, I like to point out the features of this package: (a) package instead of a notebook, with quicker loading, better balance between front and background, and available ? help, (b) embedding in $Cool and standard Mathematica packages, (c) exploit of the Mathematica programming language, with transparent code for who needs to adapt, (d) flexible input/output e.g. by use of options, (e) use of constants and functions with more arguments, (f) natural names like Genesis, Birth, etc., (g) features like Evolve, Seed, Method, Death, Mutate, FlexibleSize, Duration, (h) least squares estimation as the key example rather than centering a cart, (i) use of packages for (separate) Fitness routines, see for example the Cool`GenePro`Estimation`, Cool`GenePro`Cart` packages.
The Mathematica 3.0 version replaces Cross -> CrossBreed, and the 
GP-directory by a GenePro-directory. ChoosePointPr is added as a major 
improvement in variety.
*)

Needs["Cool`Common`"]

BeginPackage["Cool`GenePro`", Join[{"Statistics`DataManipulation`", "Statistics`DescriptiveStatistics`"}, $Cool]]

Options[Evolve] = Sort[{NumberOfGenerations -> 2, TestPrintQ -> False, GenerationQ -> True, Method -> Random}]

Options$Genesis := Sort[{MaximumLevel -> 3, Functions -> {{Plus,2}, {Times,2}}, Variables -> ToExpression[{"one", "two"}],  NumberOfIndividuals -> 5, Seed -> {}, TestPrintQ -> False}]

Options[Genesis] := Options$Genesis

Options$Genesis::usage = "ResetOptions[Genesis] will call these defaults. Note that Options[Genesis] are set by Genesis, to make the options available for the FitnessTest."

ApplyTest::usage = "ApplyTest[Population, PrintQ] calls the fitness test for each member of the population and fills the Score array. Each separate kind of problem requires its own fitness test. That test must produce a score and a hit variable. A low score means a fit individual. The hit is a printing indicator only. PrintQ is a boolean value for printing."

Birth::usage = "Birth[n] creates a new random individual with n levels. It uses options set for Genesis. Birth[] gives a random depth."

ChoosePointPr = .5

ChoosePointPr::usage = "ChoosePointPr affects Birth and CrossBreed. Basically, each individual is a formula consisting of elements at a certain position. (See PartInspect.) A random generator selects points in those formulas, and adds or switches the elements. The package has two different strategies for that random generator, and allows a random mix. If ChoosePointPr = 1, then all parts of a formula have equal probability, not just the elements but also the combinations that occur in the formula. If ChoosePointPr = 0, then the random generator cruises from the bottom to the highest level, and thereby gives more probability to the lowest elements. If for an individual the random draw <= ChoosePointPr, then the first strategy is selected. The default is .5. It appears in practice that the first strategy gives more variety while the second strategy gives more stability."

CrossBreed::usage = "CrossBreed[individual1, individual2] creates two new individuals by replacing random parts of the old ones with one another.\nCrossBreed[{i1, i2}] does the same (allowing for NestList)."

Death::usage = "The score that causes an exit from the population. Default Death is 10^10, but can be redefined in the FitnessTest option routine."
Duration::usage = "Duration[computingspeed] estimates the evolution time in minutes for the given parameters"

EvaluationPerTest::usage = "Keyword to be included in the Options of a fitness test, to be able to apply Duration, and gives the number of evaluations per test call."

Evolve::usage = "Evolve[opts___Rule] takes output of Genesis, and evolves for options FitnessTest -> ... and NumberOfGenerations -> ..."

ExeQTime::usage = "ExeQTime, option of the fitness test, an estimate of the exexution time of a single test evaluation."

Fitness::usage = "Fitness: transform 1 / (1 + Score), and then normalized so that the sum of all fitness scores is 1."

FitnessTest::usage = "FitnessTest is an Evolve Option, e.g. FitnessTest -> Estimation"

Functions::usage = "Functions: option of Genesis for: A list of functions and their number of arguments that can be used in the creation of each individual. An example is Functions -> {{Times, 2}, {Minus,1}}; and then 2 * 5 * 7 may be written as ((2 * 5) * 7). You can supply functions with 0 or more arguments (e.g. If[x, t, e, o]). You can also define a constant function, like const[] := 6, and include {const, 0}. (Another way to introduce constants is to have a variable with only constant values.)"

Genesis::usage = "Genesis[TestPrintQ -> ...] creates a new Population from scratch,  using both random depths and relationships.  Default are Options[Genesis]. TestPrintQ gives a boolean value that tells whether or not to print individuals with their number."

MaximumLevel::usage = "MaximumLevel: option of Genesis for the maximum depth any individual can be. The actual depth is determined in Birth."
Method::usage = Key[AddedUsage, Method, "Evolve option for the method of selection for procreation. Method -> Random requires fitness to be above a random chosen level, and Method -> Mean requires the score to be below average."]

Mutate::usage = "Mutate[Population] forces a new Birth if one type of individual has more than 30% of the population. If there has been a population size overflow, then the slack is also used for weeding."

Normalize::usage = "Normalize[Score] transforms Score to Fitness. A lower Score means a higher Fitness, and both are taken to be better."

NumberOfIndividuals::usage = "Option of Genesis for the number of individuals to be created"

NumberOfGenerations::usage = "Evolve Option, e.g. NumberOfGenerations -> 50"
Generation::usage = "Generation[i] contains generation i iff GenerationQ -> True"

GenerationQ::usage = "Generation[i] contains generation i iff GenerationQ -> True"

NumberOfTests::usage = "Option for the fitness test that gives the number of tests per individual. If that number is 10 and the NumberOfIndividuals -> 200, then there are 2000 tests per generation. This number helps to estimate the computing time."

Population::usage =  "Population is a list of functions. The first generation is generated by Genesis, the other generations are generated by Evolve."
Score::usage = "Score contains the basic scores of each individual."
Seed::usage = "Genesis option, to force inclusion of individuals"

TestPrintQ::usage = "TestPrintQ:\n1) Genesis option for printing of created inddividuals\n2) Evolve Option for prints of ApplyTest\n3) Option of the fitness test for prints of that test"

ToNewGeneration::usage = "ToNewGeneration[Population, forceEqualFitnessQ_:False] All data other than the population, like Fitness and Mutate, are called from the background. If forceEqualFitnessQ === True, then the Fitness array is reset to (a priori) equal fitness. (E.g. for testing.)"

Variables::usage = Key[AddedUsage, Variables, "Option of Genesis for: if Variables -> {x, y, ...}, then x, y, ... are used as possible variables for every individual"]


Begin["`Private`"]
End[]
EndPackage[]
Print["This is a crippled package: it contains only the usage statements, no routines. Use of the proper software requires a user license.
See http://www.can.nl/~cool or write to Thomas Cool, Rotterdamsestraat 69, NL-2586 GH Scheveningen, Holland for ordering details"]
